import click
import sys
import json
import awscost
from .cost_explorer import CostExplorer
from .validator import Validator
from . import constants


@click.command()
@click.option(
    "--debug/--no-debug", default=None, help="enable debug logging. (default: False)"
)
@click.option(
    "--version/--no-version", "-v", default=False, help="show version. (default: False)"
)
@click.option("--config", "-c", type=str, help="config file.")
@click.option("--profile", type=str, help="aws profile name.")
@click.option("--aws-profile", type=str, help="aws profile name.")
@click.option(
    "--granularity",
    "-g",
    type=click.Choice(["DAILY", "MONTHLY"]),
    help="granularity. (default: MONTHLY)",
)
@click.option(
    "--point",
    "-p",
    type=int,
    help=f"""duration.
if granularity is MONTHLY, {constants.DEFAULT_POINT} month ago start.
if granularity is DAILY, {constants.DEFAULT_POINT} day ago start.
(default: {constants.DEFAULT_POINT})""",
)
@click.option(
    "--start",
    callback=Validator.validate_dateformat,
    type=str,
    help=f"range of start day. default is {constants.DEFAULT_POINT} month ago.",
)
@click.option(
    "--end",
    callback=Validator.validate_dateformat,
    type=str,
    help="range of end day. default is now.",
)
@click.option(
    "--tablefmt",
    "-t",
    type=str,
    default="simple",
    help="tabulate format. (default: simple)",
)
@click.option(
    "--dimensions",
    "-d",
    type=click.Choice(constants.AVAILABLE_DIMENSIONS),
    multiple=True,
    help='group by dimensions. (default: ["SERVICE"])',
)
@click.option(
    "--filter", type=json.loads, help="filter of dimensions. default is no filter."
)
@click.option(
    "--metrics",
    type=click.Choice(constants.AVAILABLE_METRICS),
    default=constants.DEFAULT_METRICS,
    help="metrics. (default: UnblendedCost)",
)
@click.option(
    "--total/--no-total", default=True, help="include total cost. (default: True)"
)
@click.pass_context
def cli(
    ctx,
    debug,
    version,
    config,
    profile,
    aws_profile,
    granularity,
    point,
    start,
    end,
    tablefmt,
    dimensions,
    filter,
    metrics,
    total,
):
    if version:
        click.echo(awscost.VERSION)
        sys.exit()

    cost_explorer = CostExplorer(
        granularity=granularity,
        point=point,
        start=start,
        end=end,
        config=config,
        profile=profile,
        dimensions=dimensions,
        filter=filter,
        metrics=metrics,
        debug=debug,
        aws_profile=aws_profile,
        total=total,
    )
    click.echo(cost_explorer.to_tabulate(tablefmt=tablefmt))


def main():
    cli()



from boto3.session import Session
from .logger import get_logger


class CostExplorerClient:
    def __init__(
        self,
        granularity,
        start,
        end,
        filter=None,
        metrics=None,
        aws_profile=None,
        debug=False,
    ):
        self.granularity = granularity
        self.start = start
        self.end = end
        self.filter = filter
        self.metrics = metrics
        self.client = Session(profile_name=aws_profile).client(
            "ce", region_name="us-east-1"
        )
        self.logger = get_logger(debug=debug)

    def get_cost_and_usage(self, dimensions=None):
        """
        cost explorerのAPIを実行
        """
        params = self._make_params(dimensions)
        results = self.client.get_cost_and_usage(**params).get("ResultsByTime")
        self.logger.debug(results)
        return results

    def _make_params(self, dimensions):
        params = dict(
            TimePeriod={"Start": self.start, "End": self.end},
            Granularity=self.granularity,
            Metrics=[self.metrics],
        )
        group_by = self._get_group_by(dimensions)
        if group_by is not None:
            params["GroupBy"] = group_by
        if self.filter is not None:
            params["Filter"] = self.filter
        self.logger.debug(params)
        return params

    def _get_group_by(self, dimensions):
        """
        listからgroup-byの構造に変換
        """
        if dimensions is None:
            return None
        return [{"Type": "DIMENSION", "Key": key} for key in dimensions]



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
install_requires = ["tabulate", "boto3", "click>=7.0", "pyyaml"]
extras_require = {"test": ["tox", "pytest", "mock"]}

setup(
    name="awscost",
    version="0.2.5",
    description="Command Line utility for cost of aws.",
    long_description=open(os.path.join(here, "README.md")).read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="awscost tool aws",
    author="Hiroshi Toyama",
    author_email="toyama0919@gmail.com",
    url="https://github.com/toyama0919/awscost",
    license="MIT",
    packages=find_packages("src", exclude=["tests"]),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    extras_require=extras_require,
    tests_require=extras_require["test"],
    entry_points={"console_scripts": ["awscost=awscost.commands:main"]},
)
